@@ -4,8 +4,8 @@
 ;; Keywords: faces files q
 ;; Package-Requires: ((emacs "24"))
 ;; Created: 8 Jun 2015
-;; Package-Version: 20260215.425
-;; Package-Revision: d37d99d3e917
+;; Package-Version: 20260216.1450
+;; Package-Revision: 766e7b58b666
 ;; URL: https://github.com/psaris/q-mode
 
 ;; This file is not part of GNU Emacs.
@@ -43,7 +43,7 @@
 
 ;; (autoload 'q-mode "q-mode")
 
-;; The add the following to your initialization file to open all .k
+;; Then add the following to your initialization file to open all .k
 ;; and .q files with q-mode as major mode automatically:
 
 ;; (add-to-list 'auto-mode-alist '("\\.[kq]\\'" . q-mode))
@@ -66,7 +66,7 @@
 
 ;; The first q[con] session opened becomes the activated buffer.
 ;; To open a new session and send code to the new buffer, it must be
-;; actived.  Switch to the desired buffer and type `C-c M-RET' to
+;; activated.  Switch to the desired buffer and type `C-c M-RET' to
 ;; activate it.
 
 ;; Displaying tables with many columns will wrap around the buffer -
@@ -103,7 +103,7 @@
 ;; process.  This can be used to change directories before evaluating
 ;; definitions within the q process and then changing back to the root
 ;; directory.  To make the variables change values depending on which
-;; file they are sent from, values can be defined in a single line a
+;; file they are sent from, values can be defined in a single line at
 ;; the top of each .q file:
 
 ;; / -*- q-msg-prefix: "system \"d .jnp\";"; q-msg-postfix: ";system \"d .\"";-*-
@@ -168,6 +168,14 @@ each level is indented by this amount."
   :type 'string
   :group 'q)
 
+(defcustom q-flymake-on-save nil
+  "If non-nil, only run Flymake checks after saving the buffer.
+If nil, run checks for unsaved buffers by writing the current
+buffer contents to a temporary file before invoking q."
+  :safe 'booleanp
+  :type 'boolean
+  :group 'q)
+
 (defgroup q-init nil "Q initialization variables." :group 'q)
 
 (defcustom q-init-port 0
@@ -226,7 +234,6 @@ each level is indented by this amount."
 
 (defcustom q-qcon-password ""
   "Password for remote q server."
-  :safe 'stringp
   :type 'string
   :group 'q-qcon)
 
@@ -246,7 +253,8 @@ each level is indented by this amount."
 (defun q-shell-buffer-p (buffer)
   "Return non-nil if BUFFER is a live Q shell buffer.
 BUFFER can be a buffer object, buffer name, or cons cell from completion."
-  (let ((buf (get-buffer (if (consp buffer) (car buffer) buffer))))
+  (let* ((target (if (consp buffer) (car buffer) buffer))
+         (buf (and target (get-buffer target))))
     (and buf
          (buffer-live-p buf)
          (comint-check-proc buf)
@@ -265,7 +273,7 @@ Prompt with a list of live Q Shell buffers if called interactively."
   (setq q-active-buffer (get-buffer buffer)))
 
 (defun q-default-args ()
-  "Build a list of default args out of the q-init customizable variables."
+  "Build the default q command-line argument string from `q-init-*' variables."
   (concat
    (unless (equal q-init-file "") (format " %s" (shell-quote-argument q-init-file)))
    (unless (equal q-init-port 0) (format " -p %s" q-init-port))
@@ -274,7 +282,7 @@ Prompt with a list of live Q Shell buffers if called interactively."
    (unless (not q-init-garbage-collect) " -g 1")))
 
 (defun q-qcon-default-args ()
-  "Build a list of default args out of the q-init customizable variables."
+  "Build the default qcon command-line argument string from `q-qcon-*' variables."
   (concat (format "%s:%s" q-qcon-server q-qcon-port)
           (unless (equal q-qcon-user "") (format ":%s:%s" q-qcon-user q-qcon-password))))
 
@@ -308,9 +316,10 @@ command to read the command line arguments from the minibuffer."
          (host (or host ""))
          (cmd (if (equal args "") cmd (concat cmd args)))
          (qs (not (equal host "")))
-         (port (if (with-temp-buffer (setq case-fold-search nil)(string-match "-p *\\([0-9]+\\)" args)) (match-string 1 args) ""))
+         (port (let ((case-fold-search nil))
+                 (if (string-match "-p *\\([0-9]+\\)" args) (match-string 1 args) "")))
          (buffer (get-buffer-create (format "*%s*" (q-shell-name host port))))
-         (command (if qs "ssh" (getenv "SHELL")))
+         (command (if qs "ssh" (or shell-file-name (getenv "SHELL") "/bin/sh")))
          (switches (append (if qs (list "-t" host) (list "-c")) (list cmd)))
          ;; disable kdb-x rlwrap functionality
          (process-environment (cons "KX_LINE=0" process-environment))
@@ -377,7 +386,7 @@ to read the command line arguments from the minibuffer."
 This marks the PROCESS with a MESSAGE, at a particular time point."
   (comint-write-input-ring)
   (let ((buffer (process-buffer process)))
-    (setq message (substring message 0 -1)) ; strip newline
+    (setq message (replace-regexp-in-string "[\r\n]+\\'" "" (or message "")))
     (setq message (format "\nProcess %s %s at %s\n"
                           (process-name process) message (current-time-string)))
     (when (buffer-live-p buffer)
@@ -395,9 +404,13 @@ This marks the PROCESS with a MESSAGE, at a particular time point."
 
 (defun q-send-string (string)
   "Send STRING to the inferior q process stored in `q-active-buffer'."
-  (unless (cdr (assoc 'comint-process-echoes (buffer-local-variables q-active-buffer)))
-    (let ((msg (concat q-msg-prefix string q-msg-postfix)))
-      (with-current-buffer q-active-buffer
+  (unless (stringp string)
+    (user-error "Nothing to send"))
+  (unless (q-shell-buffer-p q-active-buffer)
+    (user-error "No active q buffer; run `M-x q` or activate a q shell with `C-c M-RET`"))
+  (let ((msg (concat q-msg-prefix string q-msg-postfix)))
+    (with-current-buffer q-active-buffer
+      (unless comint-process-echoes
         (goto-char (point-max))
         (insert-before-markers (concat msg "\n")))
       (comint-simple-send (get-buffer-process q-active-buffer) msg)))
@@ -424,6 +437,8 @@ This marks the PROCESS with a MESSAGE, at a particular time point."
   "Evaluate current symbol."
   (interactive)
   (let ((symbol (thing-at-point 'symbol)))
+    (unless symbol
+      (user-error "No symbol at point"))
     (q-send-string symbol)))
 
 (defun q-eval-buffer ()
@@ -473,12 +488,17 @@ This marks the PROCESS with a MESSAGE, at a particular time point."
 (defun q-eval-function ()
   "Send the current function to the inferior q[con] process."
   (interactive)
-  (save-excursion
-    (goto-char (line-end-position))          ; go to end of line
-    (let ((start (re-search-backward (concat "^" q-function-regex))) ; find beinning of function
-          (end   (re-search-forward ":")) ; find end of function name
-          (fun   (thing-at-point 'sexp))) ; find function body
-      (q-send-string (q-strip (concat (buffer-substring start end) fun))))))
+  (condition-case nil
+      (save-excursion
+        (goto-char (line-end-position))          ; go to end of line
+        (let ((start (re-search-backward (concat "^" q-function-regex))) ; find beginning of function
+              (end   (re-search-forward ":")) ; find end of function name
+              (fun   (thing-at-point 'sexp))) ; find function body
+          (unless fun
+            (user-error "Could not parse function body"))
+          (q-send-string (q-strip (concat (buffer-substring start end) fun)))))
+    (search-failed
+     (user-error "No function found around point"))))
 
 (defun q-and-go (fun)
   "Call FUN interactively and show active q buffer."
@@ -498,11 +518,13 @@ This marks the PROCESS with a MESSAGE, at a particular time point."
   (interactive)
   (q-and-go 'q-eval-region))
 
-(defun q-load-file()
+(defun q-load-file ()
   "Load current buffer's file into the inferior q[con] process after saving."
   (interactive)
+  (unless buffer-file-name
+    (user-error "Current buffer is not visiting a file"))
   (save-buffer)
-  (q-send-string (format "\\l %s" (buffer-file-name))))
+  (q-send-string (format "\\l %s" (shell-quote-argument buffer-file-name))))
 
 ;; keymaps
 
@@ -705,55 +727,70 @@ This marks the PROCESS with a MESSAGE, at a particular time point."
 (defun q-flymake (report-fn &rest _args)
   "Flymake backend using the q program.
 Takes a Flymake callback REPORT-FN as argument, as expected of a member
-of `flymake-diagnostic-functions'.  Reads contents from disk, so only
-updates when a buffer is saved."
+of `flymake-diagnostic-functions'.  q evaluates source code while
+checking it; this backend therefore performs a runtime check.
+When `q-flymake-on-save' is nil, diagnostics are produced from the
+current buffer by checking a temporary file."
   (when (process-live-p q--flymake-proc)
     (kill-process q--flymake-proc))
 
-  (let ((source (current-buffer)))
-    (save-restriction
-      (widen)
-      ;; reset the `q--flymake-proc' process to a new q process
-      (setq
-       q--flymake-proc
-       (make-process
-        :name "q-flymake" :noquery t :connection-type 'pipe
-        :buffer (generate-new-buffer " *q-flymake*")
-        :command (list q-program (buffer-file-name))
-        :sentinel
-        (lambda (proc _event)
-          ;; check that the process has exited (not just suspended)
-          (when (memq (process-status proc) '(exit signal))
-            (unwind-protect
-                ;; only proceed if `proc' is the same as
-                ;; `q--flymake-proc', which indicates that `proc' is
-                ;; not an obsolete process
-                (if (with-current-buffer source (eq proc q--flymake-proc))
-                    (with-current-buffer (process-buffer proc)
-                      (goto-char (point-min))
-                      (cl-loop
-                       while (search-forward-regexp
-                              (concat
-                               "^'[0-9.:T]* \\(.*\\)"  ; error message
-                               "\\(?:.\\|\\\n\\)*\\\n" ; stack trace
-                               "\\( *[[][0-9][]] *.*.[kq]:\\([0-9]+\\): \\).*\\\n" ; line number
-                               "\\( +^\\)$" ; carat showing column of error
-                               )
-                              nil t)
-                       for msg = (match-string 1)
-                       for prefix = (match-string 2)
-                       for row = (string-to-number (match-string 3))
-                       for carat = (match-string 4)
-                       for col = (- (length carat) (length prefix))
-                       for (beg . end) = (flymake-diag-region source row col)
-                       when (and beg end)
-                       collect (flymake-make-diagnostic source beg end :error msg)
-                       into diags
-                       finally (funcall report-fn diags)))
-                  (flymake-log :warning "Canceling obsolete check %s"
-                               proc))
-              (kill-buffer (process-buffer proc))))))) ; cleanup temp buffer
-      (process-send-eof q--flymake-proc))))
+  (let ((source (current-buffer))
+        (file (buffer-file-name)))
+    (cond
+     ((not file)
+      (funcall report-fn nil))
+     ((and q-flymake-on-save (buffer-modified-p))
+      (funcall report-fn nil))
+     (t
+      (let ((input-file file))
+        (when (buffer-modified-p)
+          (save-restriction
+            (widen)
+            (setq input-file (make-temp-file "q-flymake-" nil ".q"))
+            (write-region (point-min) (point-max) input-file nil 'silent)))
+        ;; reset the `q--flymake-proc' process to a new q process
+        (setq
+         q--flymake-proc
+         (make-process
+          :name "q-flymake" :noquery t :connection-type 'pipe
+          :buffer (generate-new-buffer " *q-flymake*")
+          :command (list q-program input-file)
+          :sentinel
+          (lambda (proc _event)
+            ;; check that the process has exited (not just suspended)
+            (when (memq (process-status proc) '(exit signal))
+              (unwind-protect
+                  ;; only proceed if `proc' is the same as
+                  ;; `q--flymake-proc', which indicates that `proc' is
+                  ;; not an obsolete process
+                  (if (and (buffer-live-p source)
+                           (with-current-buffer source (eq proc q--flymake-proc)))
+                      (with-current-buffer (process-buffer proc)
+                        (goto-char (point-min))
+                        (cl-loop
+                         while (search-forward-regexp
+                                (concat
+                                 "^'[0-9.:T]* \\(.*\\)"  ; error message
+                                 "\\(?:.\\|\\\n\\)*\\\n" ; stack trace
+                                 "\\( *[[][0-9][]] *.*\\.[kq]:\\([0-9]+\\): \\).*\\\n" ; line number
+                                 "\\( +^\\)$" ; carat showing column of error
+                                 )
+                                nil t)
+                         for msg = (match-string 1)
+                         for prefix = (match-string 2)
+                         for row = (string-to-number (match-string 3))
+                         for carat = (match-string 4)
+                         for col = (- (length carat) (length prefix))
+                         for (beg . end) = (flymake-diag-region source row col)
+                         when (and beg end)
+                         collect (flymake-make-diagnostic source beg end :error msg)
+                         into diags
+                         finally (funcall report-fn diags)))
+                    (flymake-log :warning "Canceling obsolete check %s" proc))
+                (when (and input-file (not (equal input-file file)))
+                  (ignore-errors (delete-file input-file)))
+                (kill-buffer (process-buffer proc))))))))
+        (process-send-eof q--flymake-proc)))))
 
 ;; modes
 
@@ -762,9 +799,9 @@ updates when a buffer is saved."
   "Major mode for interacting with a q interpreter."
   :syntax-table q-mode-syntax-table
   (add-hook (make-local-variable 'comint-output-filter-functions) 'comint-strip-ctrl-m)
-  (setq comint-prompt-regexp "^\\(q)+\\|[^:]*:[0-9]+>\\)")
-  (setq font-lock-defaults q-font-lock-defaults)
-  (set (make-local-variable 'comint-process-echoes) nil)
+  (setq-local comint-prompt-regexp "^\\(q)+\\|[^:]*:[0-9]+>\\)")
+  (setq-local font-lock-defaults q-font-lock-defaults)
+  (setq-local comint-process-echoes nil)
   )
 
 (defvar q-imenu-generic-expression
@@ -776,13 +813,13 @@ updates when a buffer is saved."
 (define-derived-mode q-mode prog-mode "Q-Script"
   "Major mode for editing q language files."
   :group 'q
-  (setq font-lock-defaults q-font-lock-defaults)
-  (set (make-local-variable 'comment-start) q-comment-start)
-  (set (make-local-variable 'comment-start-skip) "\\(^\\|[ \t]\\)\\(/+[ \t]*\\)")
-  (set (make-local-variable 'comment-end) "")
-  (set (make-local-variable 'indent-line-function) 'q-indent-line)
+  (setq-local font-lock-defaults q-font-lock-defaults)
+  (setq-local comment-start q-comment-start)
+  (setq-local comment-start-skip "\\(^\\|[ \t]\\)\\(/+[ \t]*\\)")
+  (setq-local comment-end "")
+  (setq-local indent-line-function 'q-indent-line)
   ;; enable imenu
-  (set (make-local-variable 'imenu-generic-expression) q-imenu-generic-expression)
+  (setq-local imenu-generic-expression q-imenu-generic-expression)
   (add-hook 'flymake-diagnostic-functions 'q-flymake nil t)
   )
 
